﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace howto_roll_two_dice
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Generate the rolls.
        private void btnRoll_Click(object sender, EventArgs e)
        {
            picGraph.Image = null;
            Cursor = Cursors.WaitCursor;
            Refresh();

            // Make an array to hold value counts.
            // The value counts[i] represents rolls of i.
            long[] counts = new long[13];

            // Roll.
            Random rand = new Random();
            long numTrials = long.Parse(txtNumTrials.Text);
            for (int i = 0; i < numTrials; i++)
            {
                int result = rand.Next(1, 7) + rand.Next(1, 7);
                counts[result]++;
            }

            // The expected percentages.
            float[] expected = 
            {
                0, 0, 1 / 36f, 2 / 36f, 3 / 36f, 4 / 36f, 5 / 36f, 
                6 / 36f, 5 / 36f, 4 / 36f, 3 / 36f, 2 / 36f, 1 / 36f
            };

            // Display the results.
            Bitmap bm = new Bitmap(
                picGraph.ClientSize.Width,
                picGraph.ClientSize.Height);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.SmoothingMode = SmoothingMode.AntiAlias;
                gr.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;
                using (StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    sf.LineAlignment = StringAlignment.Far;

                    float ymax = picGraph.ClientSize.Height;
                    float scale_x = picGraph.ClientSize.Width / 11;
                    float max_y = Math.Max(counts.Max(), numTrials * expected.Max());
                    float scale_y = ymax / (max_y * 1.05f);
                    for (int i = 2; i <= 12; i++)
                    {
                        gr.FillRectangle(Brushes.LightBlue,
                            (i - 2) * scale_x, ymax - counts[i] * scale_y,
                            scale_x, counts[i] * scale_y);
                        gr.DrawRectangle(Pens.Blue,
                            (i - 2) * scale_x, ymax - counts[i] * scale_y,
                            scale_x, counts[i] * scale_y);

                        float percent = 100 * counts[i] / (float)numTrials;
                        float expected_percent = 100 * expected[i];
                        float error = 100 * (expected_percent - percent) / expected_percent;
                        string txt = percent.ToString("0.00") +
                            Environment.NewLine +
                            expected_percent.ToString("0.00") +
                            Environment.NewLine +
                            error.ToString("0.00") + "%";
                        gr.DrawString(txt, this.Font, Brushes.Black,
                            (i - 2) * scale_x, ymax - counts[i] * scale_y);
                    }

                    // Scale the expected percentages for the number of rolls.
                    for (int i = 0; i < expected.Length; i++)
                    {
                        expected[i] *= numTrials;
                    }

                    // Draw the expected results.
                    List<PointF> expected_points = new List<PointF>();
                    expected_points.Add(new PointF(0, ymax));
                    for (int i = 2; i <= 12; i++)
                    {
                        float y = ymax - expected[i] * scale_y;
                        expected_points.Add(new PointF((i - 2) * scale_x, y));
                        expected_points.Add(new PointF((i - 1) * scale_x, y));
                    }
                    expected_points.Add(new PointF(11 * scale_x, ymax));
                    using (Pen red_pen = new Pen(Color.Red, 3))
                    {
                        gr.DrawLines(red_pen, expected_points.ToArray());
                    }
                }
            }

            picGraph.Image = bm;
            Cursor = Cursors.Default;
        }
    }
}
